<?php

namespace LakeDrops\DrupalEnvironment;

use Composer\Composer;
use Composer\IO\IOInterface;
use Composer\Plugin\Capability\CommandProvider as ComposerCommandProvider;
use Composer\Script\Event;
use Composer\Script\ScriptEvents;
use Drupal\Composer\Plugin\Scaffold\Handler as DrupalScaffoldHandler;
use LakeDrops\Component\Composer\BasePlugin;

/**
 * Composer plugin for handling drupal scaffold.
 */
class Plugin extends BasePlugin {

  /**
   * The scaffold handler object for events.
   *
   * @var \Drupal\Composer\Plugin\Scaffold\Handler
   */
  protected DrupalScaffoldHandler $scaffoldHandler;

  /**
   * {@inheritdoc}
   */
  public function activate(Composer $composer, IOInterface $io): void {
    parent::activate($composer, $io);
    if (class_exists(DrupalScaffoldHandler::class)) {
      $this->scaffoldHandler = new DrupalScaffoldHandler($composer, $io);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getHandlerClass(): string {
    return Handler::class;
  }

  /**
   * {@inheritdoc}
   */
  public function getCapabilities(): array {
    return [
      ComposerCommandProvider::class => CommandProvider::class,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      ScriptEvents::POST_CREATE_PROJECT_CMD => ['prepareProject', 1],
      ScriptEvents::POST_INSTALL_CMD => ['prepareProject', 1],
    ];
  }

  /**
   * Post update project event callback.
   *
   * @param \Composer\Script\Event $event
   *   The event that triggered the plugin.
   */
  public function prepareProject(Event $event): void {
    /** @var \LakeDrops\DrupalEnvironment\Handler $handler */
    $handler = $this->handler;
    $handler->setEvent($event);
    $handler->setupLakeDropsProject();
  }

}
